import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as mtick

def accuracy(y_actual, y_pred, y_label, nrmse):
    # set up figure and subplots
    fig, ax = plt.subplots(figsize=(14,8), nrows=2, ncols=1, gridspec_kw={'height_ratios': [3, 1]})
    
    # create plot df
    plot_df = pd.DataFrame()
    plot_df['Actual'] = y_actual
    plot_df['Predicted'] = y_pred
    plot_df['Error'] = (y_pred - y_actual) / y_actual * 100
    
    # plot actual vs predicted on grid
    plot_df[['Actual', 'Predicted']].plot(ax=ax[0], ylabel=y_label)

    ax[0].annotate(f'NRMSE = {nrmse}', xy=(0.05, 0.92), xycoords='axes fraction')

    ax[0].legend(loc="upper center", bbox_to_anchor=(0.5, 1.12), ncol=2)
    ax[0].grid(True, which='both')
    
    # plot error on grid
    plot_df[['Error']].plot(ax=ax[1], color='red')
    ax[1].grid(True, which='both')
    ax[1].legend(loc="upper center", bbox_to_anchor=(0.5, 1.35), ncol=2)
    fmt = '%.0f%%' # Format you want the ticks, e.g. '40%'
    yticks = mtick.FormatStrFormatter(fmt)
    ax[1].yaxis.set_major_formatter(yticks)
    
    # show plots
    fig.autofmt_xdate(rotation=45)
    plt.gcf().suptitle("Actual vs Predicted", fontsize=20)
    
    plt.show()


import numpy as np
from sklearn import metrics

def nrmse(y_actual, y_pred):
    # normalized root mean square error
    return round(np.sqrt(metrics.mean_squared_error(y_actual, y_pred)) / np.mean(y_actual), 3)


import pandas as pd
import statsmodels.api as sm
from sklearn.model_selection import train_test_split
from typing import List

from . import bfe
from . import nrsme

def model(df:pd.DataFrame, date_label:str, y_label:str, X_labels:List[str]) -> None:
    """Builds a Marketing Mix Model
    Params:
    - df: Pandas DataFrame - contains your data (weekly Mon->Sun)
    - date_label: str - the date (YYYY-MM-DD) at the start of the week (Mon)
    - y_label: str - the dependent variable or target (sales, revenue, leads, etc)
    - X_labels: List[str] - a list of explanatory variables (media spend, seasonality, etc)
    
    Results:
    - prints model
    - displays charts
    """
    # keep only the columns we need
    keep_columns = [y_label, date_label] + X_labels

    # make a copy of the dataframe so we don't change the original
    model_df = df[keep_columns].copy()

    # split out the X_data and y_data
    y_data = model_df[y_label]
    X_data = model_df[X_labels]

    # add a constant to the X_data
    X_data = sm.add_constant(X_data)

    # generate train test splits
    X_train, X_test, y_train, y_test = train_test_split(X_data, y_data,
        test_size=0.2, random_state=1130)

    # run backwards feature elimination (BFE) to get only important features
    important_features = bfe(y_train, X_train)

    # run model with only important_features
    X_train_if = X_train[important_features]
    est = sm.OLS(y_train, X_train_if).fit()

    # predict y_test with model
    X_test_if = X_test[important_features]
    y_pred = est.predict(X_test_if)

    # get model coefficients and p_values
    coefs = est.params.values
    p_values = est.p_values.values

    # get the normalized root mean square error
    nrmse = nrsme(y_test, y_pred)

    print(est.summary())


"""The setup script."""
import pathlib
from setuptools import setup, find_packages
HERE = pathlib.Path(__file__).parent
VERSION = "0.0.0.2"
PACKAGE_NAME = "hommmer"
AUTHOR = "mike taylor"
AUTHOR_EMAIL = "mike@saxifrage.xyz"
URL = "https://github.com/hammer-mt/hommmer"
LICENSE = "MIT"
DESCRIPTION = "A simple Marketing Mix Modeling library in Python"
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf8')
LONG_DESC_TYPE = "text/markdown"
INSTALL_REQUIRES = [
    "numpy",
    "pandas",
    "matplotlib",
    "statsmodels",
    "typing",
    "sklearn"
]
setup(name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    license=LICENSE,
    author_email=AUTHOR_EMAIL,
    url=URL,
    install_requires=INSTALL_REQUIRES,
    package_dir={"": "src"},
    packages=find_packages(where="src")
    )
